#####################################
##
## 第２章 データ加工
##
##
## Author: Yohei Sato
##
#####################################


##
## 2.2
##

head(airquality)

# subset関数
head(subset(airquality, Month==5))

# filter関数
library(dplyr)

airquality %>% filter(Month==5) %>% head()


# 使用するデータ
head(mtcars)


# 大小関係
head(subset(mtcars, mpg>=20))


head(subset(mtcars, mpg<20))


# 複数条件
mtcars %>% filter(mpg>=20 & cyl>=6) %>% head()

mtcars %>% filter(mpg>=20 | cyl>=6) %>% head()


# 値が含まれているものを抽出
head(subset(mtcars, carb %in% c(1,3)))


##
## 2.3
##


# Ozoneで昇順
head(arrange(airquality, Ozone))


# Ozoneで降順
head(arrange(airquality, desc(Ozone)))


# Ozoneで昇順、Solar.Rで降順
head(arrange(airquality, Ozone, desc(Solar.R)))

##
## 2.4
##

(a <- head(airquality, 3))


(b <- tail(airquality, 3))


(ab <- rbind(a, b))


library(dplyr)

df <- data.frame(Ozone = 33)


tail(rbind_all(list(airquality, df)))


a <- data.frame(id=1:10, id2=11:20, x=101:110)
b <- data.frame(id=c(1,3,5,11), id2=c(11, 17, 15, 24), y=c(100, 200, 150, 400))


# 内部結合
ab1 <- merge(a, b, by="id")
ab1

# 複数のキーを指定して結合
ab1_2 <- merge(a, b, by=c("id", "id2"))
ab1_2

# 同じ名前の変数があったときに、変数名の後につける文字の指定
ab1_3 <- merge(a, b, by="id", suffixes=c(".a", ".b"))
ab1_3

# 左側のデータを全部残す結合（left_join)
ab2 <- merge(a, b, by="id", all.x=T)
ab2

# 右側のデータを全部残す結合（right_join）
ab3 <- merge(a, b, by="id", all.y=T)
ab3

# 全部のデータを残す結合（full_join）
ab4 <- merge(a, b, by="id", all=T)
ab4


a <- data.frame(id=as.integer(1:10), id2=as.integer(11:20), x=101:110)
b <- data.frame(id=as.integer(c(1,3,5,11)), id2=as.integer(c(11,17,15,24)), y=c(100, 200, 150, 400))


# 内部結合
ab1 <- inner_join(a, b, by="id")
ab1


# 複数のキーを指定して結合
ab1_2 <- inner_join(a, b, by=c("id","id2"))
ab1_2


# 左側のデータを全部残す結合
ab2 <- left_join(a, b, by="id", all.x=T)
ab2


##
## 2.5
##

library(reshape2)

head(airquality)


airquality.melt <- melt(airquality, id=c("Month", "Day"))
head(airquality.melt)

airquality.melt <- melt(airquality, id=c("Month", "Day"),
                        variable.name="air", value.name="val")

head(airquality.melt)


aqm <- melt(airquality, id=c("Month", "Day"), na.rm=TRUE)
dcast(aqm, Month ~ variable, mean)

dcast(aqm, Month ~ variable, mean, margins = "Month")

dcast(aqm, Month ~ variable, mean, margins = "variable")

dcast(aqm, Month ~ variable, mean, margins = c("Month","variable"))

dcast(aqm, Month ~ variable, mean, margins = T)

library(plyr)

dcast(aqm, Month ~ variable, mean, subset = .(variable == c("Ozone", "Temp")))


head(dcast(aqm, Month + Day ~ variable, mean, 
           subset = .(variable == c("Ozone", "Temp"))))


head(dcast(aqm, Month ~ variable + Day, mean, 
           subset = .(variable == c("Temp"))), n=3)




##
## 2.6
##

library(dplyr)

head(mtcars)

# 一つの集約変数ごとに平均を求める
mtcars %>%
  group_by(am) %>%
  summarise(mpg.mean=mean(mpg))


# 複数の集約変数ごとに平均を求める
mtcars %>%
  group_by(am, vs) %>%
  summarise(mpg.mean=mean(mpg))


mtcars %>%
  group_by(am, vs) %>%
  summarise(cyl.max=cyl[which.max(mpg)])


mtcars %>%
  group_by(am, vs) %>%
  summarise(mpg.mean=mean(mpg),
            disp.mean=mean(disp))


# 集約結果をおとのデータに追加する
mtcars %>%
  group_by(am, vs) %>%
  mutate(mpg.mean=mean(mpg),
         disp.mean=mean(disp)) %>%
  head()



##
## 2.7
##

# データフレームのリスト作成
l <- list(airquality[1:3,], airquality[11:13,], airquality[21:23,])
l

# リストをデータフレームに変換
df <- rbind_all(l)
df

library(foreach)

date.from <- as.Date("2014-04-01")
date.to <- as.Date("2014-04-03")

df2 <- rbind_all(foreach(day=seq.Date(date.from, date.to, "days"), combine=rbind) %do% {
  f <- sprintf("aq_%s.csv", day)
  read.csv(f, header=T, stringsAsFactors=F)
})

df2



##
## 2.8
##

aq <- airquality

names(aq)


names(aq)[3] <- "wind"
names(aq)


names(aq)[names(aq)=="Temp"] <- "temp"
names(aq)

head(rename(airquality, replace = c(Ozone = "Oooozone")))


x <- c("a", "b", "c")
x <- ifelse(x=="a", "AAA",
            ifelse(x=="c", "CCC", x))
x


x <- c("a", " b", "c")
mapvalues(x, c("a", "c"), c("AAA", "CCC"))
revalue(x, c(a = "AAA", c = "CCC"))


round(135 / 10) * 10

floor(135 / 10) * 10

ceiling(135 / 100) * 100

round_any(135, 10)
round_any(135, 10, floor)
round_any(135, 100, ceiling)


d_ply(airquality, .(Month), function(x){
  month <- x$Month[1]
  write.csv(x, sprintf("airquality_%s.csv", month))
})
