#####################################
##
## 第３章 データ可視化
##
##
## Author: Yohei Sato
##
#####################################


##
## 3.1 仮想データの作成
##


# 商品1の月次売上データの作成
sales.data1 <- data.frame(
  item="item1",
  m=4:9,
  month=paste0("2014-0",c(4:9)),
  sales=c(800,1000,1800,1500,1400,2000))

sales.data1


# 商品1,2の月次売上データの作成
sales.data2 <- data.frame(
  item="item2",
  m=4:9,
  month=paste0("2014-0",c(4:9)),
  sales=c(700,900,1500,1700,1800,2400))
sales.data12 <- rbind(sales.data1, sales.data2)
sales.data12

  
# 店舗1〜3の商品3,4の日次売上データの作成
set.seed(10)
sales.data34 <- data.frame(
  shop=sample(1:3,100,replace=T),
  item3=rnorm(100,100,30)
)

sales.data34$item4 <- round(0.9 * sales.data34$item3 + 
                              80 * sales.data34$shop + 
                              rnorm(100,30,15))

sales.data34$shop <- paste0("shop",sales.data34$shop)
head(sales.data34)

  
##
## 3.2 棒グラフ
##

library(ggplot2)

## 基本
ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity")


# 背景色を白にする
ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity") +
  theme_bw()


# グラフタイトルとラベルの設定
ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  theme_bw()

# 日本語のフォントを指定する
base_family <- "Osaka"

ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  theme_bw(base_family=base_family)

# 自分用のテーマ関数を作成する
my_theme <- function() {
    theme_bw(base_family="Osaka") # フォントの設定
}

  
# 数値のフォーマットの変更
library(scales)

ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  my_theme()


# 棒の上に値を表示する
ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity") +
  geom_text(aes(label=sales), vjust=-0.2) +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  my_theme()

# 数値のフォーマットを3ケタ区切りにする
ggplot(sales.data1, aes(x=month, y=sales)) +
  geom_bar(stat="identity") +
  geom_text(aes(label=comma(sales)), vjust=-0.2) +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  my_theme()


# 積み上げ棒グラフの作成
ggplot(sales.data12, aes(x=month, y=sales, fill=item)) +
  geom_bar(stat="identity") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  my_theme()


# 色セットの変更
library(RColorBrewer)
rownames(brewer.pal.info)


ggplot(sales.data12, aes(x=month, y=sales, fill=item)) +
  geom_bar(stat="identity") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  scale_fill_brewer(palette = "Paired") +
  my_theme()


# 棒グラフに枠を付ける
ggplot(sales.data12, aes(x=month, y=sales, fill=item)) +
  geom_bar(stat="identity", col="black") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  scale_fill_brewer(palette = "Paired") +
  my_theme()


# 100％積み上げグラフの作成
ggplot(sales.data12, aes(x=month, y=sales, fill=item)) +
  geom_bar(position="fill",stat="identity", col="black") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=percent) +
  scale_fill_brewer(palette = "Paired") +
  my_theme()


# 棒グラフを横に並べる
ggplot(sales.data12, aes(x=month, y=sales, fill=item)) +
  geom_bar(position="dodge",stat="identity",col="black") +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  scale_fill_brewer(palette = "Paired") +
  my_theme()

##
## 3.3 折れ線グラフ
##

# 基本的な使い方
## xが数値の時の折れ線グラフ
ggplot(sales.data1, aes(x=m, y=sales)) +
  geom_line() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  my_theme()

## xがファクター型の時の折れ線グラフ
ggplot(sales.data1, aes(x=month, y=sales,group=1)) +
  geom_line() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma) +
  my_theme()


# y軸の範囲を指定する
ylimit <- c(0,max(sales.data1$sales))
ggplot(sales.data1, aes(x=month, y=sales,group=1)) +
  geom_line() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma,limits=ylimit) +
  my_theme()


# 折れ線グラフに点を追加する
ggplot(sales.data1, aes(x=month, y=sales,group=1)) +
  geom_line() +
  geom_point() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma,limits=ylimit) +
  my_theme()


# 折れ線グラフをグループ化する
ylimit <- c(0,max(sales.data12$sales))
ggplot(sales.data12, aes(x=month, y=sales,group=item,col=item)) +
  geom_line() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma,limits=ylimit) +
  my_theme()

# グループごとに線の種類を変える
ggplot(sales.data12, aes(x=month, y=sales,group=item,col=item,lty=item)) +
  geom_line() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma,limits=ylimit) +
  my_theme()

# グループごとに線と点の種類を変える
ggplot(sales.data12, aes(x=month,y=sales,group=item,col=item,lty=item,ggplshape=item)) +
  geom_line() +
  geom_point() +
  xlab("月") +
  ylab("売上") +
  ggtitle("売上推移") +
  scale_y_continuous(label=comma,limits=ylimit) +
  my_theme()


##
## 3.4 散布図
##

# 基本的な使い方
ggplot(sales.data34, aes(x=item3, y=item4)) +
  geom_point() +
  xlab("item3") +
  ylab("item4") +
  ggtitle("item3とitem4の散布図") +
  scale_y_continuous(label=comma) +
  my_theme()

# 塗りつぶしなしの散布図
ggplot(sales.data34, aes(x=item3, y=item4)) +
  geom_point(shape=21) +
  xlab("item3") +
  ylab("item4") +
  ggtitle("item3とitem4の散布図") +
  scale_y_continuous(label=comma) +
  my_theme()


# 散布図をグループ化する（グループごとに色と形を変える）
ggplot(sales.data34, aes(x=item3, y=item4, col=shop, shape=shop)) +
  geom_point() +
  xlab("item3") +
  ylab("item4") +
  ggtitle("item3とitem4の散布図") +
  scale_y_continuous(label=comma) +
  my_theme()


# 回帰直線の追加
ggplot(sales.data34, aes(x=item3, y=item4)) +
  geom_point() +
  geom_smooth(method="lm") +
  xlab("item3") +
  ylab("item4") +
  ggtitle("item3とitem4の散布図") +
  scale_y_continuous(label=comma) +
  my_theme()



# グループごとに回帰直線を引く
ggplot(sales.data34, aes(x=item3, y=item4, col=shop, shape=shop, group=shop)) +
  geom_point() +
  geom_smooth(method="lm") +
  xlab("item3") +
  ylab("item4") +
  ggtitle("item3とitem4の散布図") +
  scale_y_continuous(label=comma) +
  my_theme()

##
## 3.5 ヒストグラム
##

# 基本的な使い方
ggplot(sales.data34, aes(x=item3)) +
  geom_histogram() +
  ggtitle("item3の売上分布") +
  scale_y_continuous(label=comma) +
  my_theme()


# 棒に枠を付けて色を白にする
ggplot(sales.data34, aes(x=item3)) +
  geom_histogram(fill="white",col="black") +
  ggtitle("item3の売上分布") +
  scale_y_continuous(label=comma) +
  my_theme()


# ビン幅を変更する 
## ビン幅を10に設定
ggplot(sales.data34, aes(x=item3)) +
  geom_histogram(fill="white",col="black",binwidth=10) +
  ggtitle("item3の売上分布") +
  scale_y_continuous(label=comma) +
  my_theme()


# ヒストグラムをグループ化する
## グループごとに色を付ける
ggplot(sales.data34, aes(x=item3,fill=shop)) +
  geom_histogram(position="dodge",col="black",binwidth=10) +
  ggtitle("item3の売上分布") +
  scale_y_continuous(label=comma) +
  scale_fill_brewer(palette = "Paired") +
  my_theme()


## グループごとに複数のヒストグラムを描く
ggplot(sales.data34, aes(x=item3)) +
  geom_histogram(fill="white",col="black",binwidth=10) +
  ggtitle("item3の売上分布") +
  scale_y_continuous(label=comma) +
  facet_grid(shop~.) +
  my_theme()


##
## 3.6 箱ひげ図
##

# 基本体な使い方
ggplot(sales.data34, aes(x=shop,y=item3)) +
  geom_boxplot() +
  ggtitle("店舗ごとのitem3の売上分布") +
  scale_y_continuous(label=comma) +
  my_theme()


# 箱ひげ図をグループ化する
library(reshape2)
sales.data34.melt <- 
  melt(sales.data34,id.vars=c("shop"),value.names=c("item3","item4"))

ggplot(sales.data34.melt, aes(x=variable,y=value)) +
  geom_boxplot() +
  xlab("") +
  ylab("売上") +
  ggtitle("店舗ごとのitem3,4の売上分布") +
  scale_y_continuous(label=comma) +
  facet_grid(.~shop) +
  my_theme()
