#####################################
##
## 第４章 問題発見
##
##
## Author: Yohei Sato
##
#####################################

#####################################
##
## 4.3 基礎集計で問題を見つける
##
#####################################


##
## 4.3.1 インパクトを比較する
##

payment.daily <- read.csv("payment_daily1.csv", header=T, stringsAsFactors=F)
head(payment.daily)

  
# 月のカラムを追加
payment.daily$log_month <- paste0(substr(payment.daily$log_date, 1, 7),"-01")


library(dplyr) 
#  注意
#   plyrと同時に使用すると関数名がぶつかっておかしくなる
#   結果がおかしい場合には、detach("package:plyr") を実行して下さい。

payment.monthly <- 
  payment.daily %>%
  group_by(log_month, item_name) %>%
  summarise(payment=sum(payment))

head(payment.monthly)



library(ggplot2)
library(scales)

my_theme <- function(){
  theme_bw(base_family="Osaka")
}


# 実数の推移
## グラフ
ggplot(payment.monthly, aes(x=log_month, y=payment, fill=item_name)) + 
  geom_bar(stat="identity",position="dodge",col="black") +
  scale_y_continuous(label=comma) +
  xlab("月") + 
  ylab("売上") +
  scale_fill_brewer(palette = "Paired") +
  my_theme()

## 表
library(reshape2)

payment.monthly.cast <- dcast(payment.monthly, item_name ~ log_month)
payment.monthly.cast



# 構成比率の推移
## グラフ
ggplot(payment.monthly, aes(x=log_month, y=payment, fill=item_name)) + 
  geom_bar(stat="identity",position="fill",col="black") +
  scale_y_continuous(label=percent) +
  xlab("月") + 
  ylab("売上構成") +
  scale_fill_brewer(palette = "Paired") +
  my_theme()

## 表
payment.monthly.cast[, -1] <- round(payment.monthly.cast[,-1]/colSums(payment.monthly.cast[, -1]), 2)
payment.monthly.cast


##
## 4.3.2 分解して比較する
##

payment.daily <- read.csv("payment_daily2.csv", header = T,stringsAsFactors = F)
head(payment.daily)


# 月のカラムを追加
payment.daily$log_month <- substr(payment.daily$log_date, 1, 7)

library(dplyr)

payment.monthly <- 
  payment.daily %>%
  group_by(log_month) %>%
  summarise(
    arpu=sum(payment) / length(unique(user_id)),
    payment=sum(payment),
    uu=n_distinct(user_id))

payment.monthly



# 1人当たり売上推移
ylimit <- c(0, max(payment.monthly$arpu))
ggplot(payment.monthly, aes(x = log_month, y = arpu, group = 1)) +
  geom_point() + 
  geom_line() + 
  scale_y_continuous(label = comma, limits = ylimit) +
  xlab("月") + 
  ylab("1人当たり売上") + 
  my_theme()


# 購入者数推移
ylimit <- c(0, max(payment.monthly$uu))
ggplot(payment.monthly, aes(x = log_month, y = uu, group = 1)) +
  geom_point() + 
  geom_line() + scale_y_continuous(label = comma, limits = ylimit) +
  xlab("月") + 
  ylab("購入者数") + 
  my_theme()


user.info <- read.csv("user_info.csv", header = T,stringsAsFactors = F)

# 登録月の追加
user.info$reg_month <- substr(user.info$regist_date, 1, 7)
head(user.info)


payment.daily <- merge(payment.daily, user.info, by = "user_id")
head(payment.daily)


payment.daily$user_type <- ifelse(payment.daily$log_month == payment.daily$reg_month, "new", "existing")

payment.monthly2 <- 
  payment.daily %>%
  group_by(log_month, user_type) %>%
  summarise(
    arpu=sum(payment)/length(unique(user_id)),
    payment = sum(payment), 
    uu = n_distinct(user_id))

head(payment.monthly2)


# 一人当たり売上推移
ylimits <- c(0, max(payment.monthly2$arpu))
ggplot(payment.monthly2, aes(x = log_month, y = arpu, col = user_type, group = user_type, lty=user_type, shape=user_type)) + 
  geom_line() + 
  geom_point() + 
  scale_y_continuous(label = comma,limits = ylimits) + 
  xlab("月") + 
  ylab("1人当たりの売上") + 
  my_theme()



# 購入者数推移
ggplot(payment.monthly2, aes(x = log_month, y = uu, fill = user_type, group = user_type)) + 
  geom_bar(stat = "identity", col = "black") +
  scale_fill_brewer(palette = "Paired") +
  scale_y_continuous(label = comma) + 
  xlab("月") + 
  ylab("購入者数") + 
  my_theme()


##
## 4.3.3 トレンドを見る
##

dau <- read.csv("daily_active_user1.csv", header = T, stringsAsFactors = F)
head(dau)


dau$log_month <- substr(dau$log_date, 1, 7)

dau.summary1 <- 
  dau %>%
  group_by(log_month) %>%
  summarise(
    mau=n_distinct(user_id))
head(dau.summary1)


my_theme <- function() {
  theme_bw(base_family = "Osaka") + 
    theme(axis.text.x =element_text(angle = 45, vjust = 0.5))
}


ggplot(dau.summary1, aes(x = log_month, y = mau)) + 
  geom_bar(stat = "identity") + 
  scale_y_continuous(label = comma) + 
  xlab("") + 
  ylab("MAU") +
  ggtitle("MAU推移") + 
  my_theme()


# ユーザーの登録情報の読み込み
user.info <- read.csv("user_info2.csv", header = T, stringsAsFactors = F)
head(user.info)


dau <- merge(dau, user.info, by = "user_id")
head(dau)

# 男女別MAU
dau.gender.summary <-
  dau %>%
  group_by(log_month, gender) %>%
  summarise(
    mau=n_distinct(user_id))


ylimit <- c(0, max(dau.gender.summary$mau))
ggplot(dau.gender.summary, aes(x=log_month, y=mau, col=gender, group=gender,lty=gender, shape=gender)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("MAU") +
  ggtitle("男女別MAU推移") +
  scale_y_continuous(label=comma, limits=ylimit) +
  my_theme()

# 年代別MAU
dau.generation.summary <-
  dau %>%
  group_by(log_month, generation) %>%
  summarise(
    mau=n_distinct(user_id))



dau.generation.summary$generation <- as.factor(dau.generation.summary$generation)
ylimit <- c(0, max(dau.generation.summary$mau))
ggplot(dau.generation.summary, aes(x=log_month, y=mau, col=generation, group=generation, lty=generation, shape=generation)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("MAU") +
  ggtitle("年代別MAU推移") +
  scale_y_continuous(label=comma, limits=ylimit) +
  my_theme()


# 性別×年代別MAU
dau.demogra.summary <- 
  dau %>%
  group_by(log_month, gender, generation) %>%
  summarise(
    mau=n_distinct(user_id))

dau.demogra.summary$generation <- as.factor(dau.demogra.summary$generation)
ylimit <- c(0, max(dau.demogra.summary$mau))
ggplot(dau.demogra.summary, aes(x=log_month, y=mau, col=generation, group=generation, lty=generation, shape=generation)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("MAU") +
  ggtitle("男女別年代別MAU推移") +
  scale_y_continuous(label=comma, limits=ylimit) +
  facet_grid(gender~.) +
  my_theme()


# デバイス別MAU
dau.device.summary <- 
  dau %>%
  group_by(log_month, device) %>%
  summarise(
    mau=n_distinct(user_id))

ylimit <- c(0, max(dau.device.summary$mau))
ggplot(dau.device.summary, aes(x=log_month, y=mau, col=device, group=device, lty=device, shape=device)) +
  geom_line() +
  geom_point() +
  xlab("") +
  ylab("MAU") +
  ggtitle("デバイス別MAU推移") +
  scale_y_continuous(label=comma, limits=ylimit) +
  my_theme()


#####################################
##
## 4.4 データマイニングで問題を見つける
##
#####################################


intention.data <- read.csv("intention2.csv", header = T,
                           stringsAsFactors = F, fileEncoding = "UTF-8")
head(intention.data)


library(rpart)
library(partykit)

# 利用意向をfactor型に変換
intention.data$intention <- as.factor(intention.data$intention)


# id以外の全部の変数を使う
keys <- names(intention.data)[!names(intention.data) %in% c("id")]
intention.rp0 <- rpart(intention ~ ., data = intention.data[, keys], method = "class")
print(intention.rp0)

plot(as.party(intention.rp0))



# 複雑度パラメータの最小値をデフォルトの0.01よりも小さくして実行
intention.rp1 <- rpart(intention ~ ., data = intention.data[, -1],
                       method = "class", cp = 0.001)
print(intention.rp1)

plot(as.party(intention.rp1))

# 決定木の結果を使ってクロス集計
intent.data2 <- intention.data[, c("A29", "A8", "A28", "A26","intention")]
intent.data2$A29 <- ifelse(intent.data2$A29 >= 0.5, "0.5〜", "〜0.5")
intent.data2$A8 <- ifelse(intent.data2$A8 >= 0.5, "0.5〜", "〜0.5")
intent.data2$A28 <- ifelse(intent.data2$A28 >= 0.5, "0.5〜", "〜0.5")
intent.data2$A26 <- ifelse(intent.data2$A26 >= 1.5, "1.5〜", "〜1.5")


intent.data2$intention <- as.numeric(as.character(intent.data2$intention))
intent.data2.summary <- 
  intent.data2 %>%
  group_by(A29,A8,A28,A26) %>%
  summarise(intention.rate=sum(intention)/length(intention)) %>%
  arrange(desc(intention.rate))

intent.data2.summary



